@@ -7,6 +7,9 @@ gi = GeoIP.new(GeoIP.GEOIP_MEMORY_CACHE)
 #gi = GeoIP.open("/usr/local/share/GeoIP/GeoIP.data",GeoIP.GEOIP_STANDARD)
 
 print gi.country_code_by_name("yahoo.com")
-print gi.country_name_by_name("www.government.de")
-print gi.country_code_by_addr("203.195.93.0")
-print gi.country_name_by_addr("203.195.93.0")
+print gi.last_netmask()
+print gi.country_name_by_name("www.bundestag.de")
+print gi.country_code_by_addr("24.24.24.24")
+print gi.country_name_by_addr("24.24.24.24")
+print gi.range_by_ip("68.180.206.184")
+
