@@ -11,10 +11,12 @@ if gir != None:
 	print gir['country_code']
 	print gir['country_code3']
 	print gir['country_name']
-	print gir['region']
 	print gir['city']
+	print gir['region']
+	print gir['region_name']
 	print gir['postal_code']
 	print gir['latitude']
 	print gir['longitude']
-	print gir['dma_code']
 	print gir['area_code']
+	print gir['time_zone']
+	print gir['metro_code']
